;******************************************************************************************
;    COMPOSANT	:  ATMEL AVR 8 Bits (RISC)       AT90S2313 	Quartz = 3,6864 MHz
;    PROGRAMME	:  Terminal.ASM
;    VERSION	:  V1.0
;    DATE		:  16/07/00
;    DERNIERE MAJ.	:  30/07/00
;    DESCRIPTION	:  Terminal RS232 : LCD 2x16 Car + Clavier 16 touches
;			:  format 1200,n,8,1
;				
;    AUTEUR	:  Igor GODI 	Email : igor.godi@freesbee.fr
;    TESTS	:  OK
;******************************************************************************************
;			DIRECTIVES D'ASSEMBLAGE
;******************************************************************************************
	.device 	AT90S2313
	.include	"2313def.inc"	; Fichier de dfinition

;******************************************************************************************
;			CONSTANTES
;******************************************************************************************
;.equ xxxxx = xxxxxx


;******************************************************************************************
;			PORTS D'ENTREES/SORTIES
;******************************************************************************************
;________________________ CONFIGURATION PORT B __________________________________________
; Port B :	(I/O)	=	Liaison		; Description
;-------
; B_0	(Out)	=	DO LCD		; Fils rlis au bus du LCD
; B_1	(Out)	=	D1 LCD		
; B_2	(Out)	=	D2 LCD		; Et au clavier Matric
; B_3	(Out)	=	D3 LCD		
; B_4	(Out)	=	D4 LCD		
; B_5	(Out)	=	D5 LCD		
; B_6	(Out)	=	D6 LCD		
; B_7	(Out)	=	D7 LCD		
;			  76543210
.equ	CONFIGB	  =	0b11111111

;________________________ CONFIGURATION PORT D __________________________________________
; Port D :	(I/O)	=	Liaison		; Description
;-------
; D_0	(in)	=	Rx_RS232	; Fil Rx RS232
; D_1	(out)	=	Tx_RS232	; Fil Tx RS232
; D_2	(xxx)	=	xxxxxxxx	; Fil xxxxxxxxx
; D_3	(xxx)	=	xxxxxxxx	; Fil xxxxxxxxx
; D_4	(out)	=	Fil E_LCD	; Fil Signal Enable LCD
; D_5	(out)	=	Fil RS_LCD	; Fil Signal Register Select LCD
; D_6	(xxx)	=	xxxxxxxx	; Fil xxxxxxxxx
;			  -6543210
.equ	CONFIGD	  =	0b00110010

;******************************************************************************************
;			AUTRES CONNEXIONS ELECTRIQUES
;******************************************************************************************
; Pour Le LCD :
;	* R/W LCD  la masse
;	* +5V LCD au +5Valim
;	* Masse LCD  la masse alim

;******************************************************************************************
;			DEFINITIONS DE REGISTRES
;******************************************************************************************
; Registres  usage Global :
;-------------------------
.def DRAPEAUX	= r23	; Registre destin  contenir des drapeaux (Commun LCDCLAV.SUB)
			; B7 (Ou LCD_Occupe) 0:Bus LCD Libre  ou   1: Bus LCD Occup
			; B6 Permet d'viter la rptition 	0: Pas de touche frapp avant
			;					1: Dja frapp
			; B5 OrdiOn   	0: Ordi N'envoi pas signe de vie
			;		1: Ordi en ligne (Chien de garde ractiv avant 10s)
			; B4  B0  dfinir par utilisateur

.def Touche	= r22	; Registre qui contiendra le code rel de la touche enfonce du clavier
.def T10	= r21	; Diviseur pour Timer 10s
.def AfficheTps = r20   ; Temps d'affiche message  l'ecran

; Registres  usage Local :
;------------------------

.def zh		= r31		; Pointeur Z  (Utilis pour pointer en FLASH)
.def zl		= r30

.def temp	= r16		; Registre temporaire


;******************************************************************************************
;			ORGANISATION RAM
;******************************************************************************************
.dseg
.org 	0x60
;xxxxx: 		byte x

;******************************************************************************************
;			ORGANISATION EEPROM
;******************************************************************************************
.eseg
.org	0x00
  ;	xxxx:	.DB	"xxxxx"


;******************************************************************************************
;			INTERRUPTIONS ET RESET
;******************************************************************************************
.cseg
.org	0x00
	rjmp 	RESET		; Vecteur RESET
	reti			; Interruption Externe 0
	reti			; Interruption Externe 1
	reti			; Capture Timer1
	rjmp	T1_COMP		; Comparaison Timer1
	reti			; Timer1 Overflow
	reti			; Timer0 Overflow
	rjmp	UART_RXC	; Reception RS232 complte
	reti			; Registre emission RS232 vide
	reti			; Emission RS232 complte
	reti			; Comparateur Analogique

;******************************************************************************************
;			PROGRAMME PRINCIPAL (RESET)
;******************************************************************************************
RESET :
;__________________________________ INITIALISATIONS _______________________________________
	;----------------- Initialisation de la pile --------------------------------------
	ldi 	temp,low(RAMEND)	; Pointe en fin de mmoire
	out 	SPL,temp

	;----------------- Initialisation des ports ---------------------------------------
	ldi	temp, CONFIGB	; Init PORT B voir au dessus pour dfinition des broches
	out	DDRB,temp
	ldi	temp, CONFIGD	; Init PORT D voir au dessus pour dfinition des broches
	out	DDRD,temp

	;----------------- Initialisation des Timer (0 et 1) ------------------------------
		; TIMER 1 :
	ldi 	temp, high(0x00B4)	; Poid fort registre de comparaison (si utilis)
	out	OCR1AH,temp
	ldi 	temp, low(0x00B4)	; Poid faible registre de comparaison (si utilis)
	out	OCR1AL,temp		; Soit 50ms
	ldi	temp,0x00		; Utilisation de la broche de sortie 0C1 : Non
					; PWM : Arrt
	out 	TCCR1A,temp		
	ldi	temp,0x0D		; Rducteur de bruit : Non
					; Mode Comparaison (TCNT1=OCR=>RAZ) : Marche
					; Source horloge : Ck/1024=3600Hz
	out 	TCCR1B,temp		
		; INTERRUPTIONS DU TIMER
	ldi 	temp,0x40		; Timer1 : Comparaison Activ 
					
	out 	TIMSK,temp

	;----------------- Initialisation de la RS232 --------------------------------------
	ldi	temp,0xBF			; Slection de la vitesse : 1200 bauds 
					; (Quartz = 3,6864 MHz)
	out 	UBRR,temp
	ldi	temp,0x98		; Tx : Autoris 
					; Rx : Autoris 
					; Mode : 8 bits
					; Valeur 9me bit : x (0 ou 1)
					; Interruption RxC : Active
					; Interruption UDRE :pas Active 
					; Interruption TxC : pas Active
	out 	UCR,temp

	;----------------- Initialisation Comparateur Analogique ---------------------------
	ldi 	temp,0x80		; Comparateur Analogique : Arrt
	out	ACSR,temp

	;----------------- Initialisation des registres ------------------------------------
	clr Drapeaux			; OrdiOn=0
	clr T10				; chien de garde  0
	clr AfficheTps			; AfficheTps=0

	;----------------- Programmes d'Initialisation -------------------------------------
	rcall	INIT_LCD		; Initialisation Afficheur LCD
	rcall	INIT_CONFIG_LCD		; 2 lignes mode 8 bits

;__________________________________ PROGRAMME PRINCIPAL ________________________________
	ldi zl,low(BV1*2)		; Afficher la ligne 1 Bienvenue
	ldi zh,high(BV1*2)	
	rcall Aff_Lg1

	ldi zl,low(BV2*2)		; Afficher la ligne 2 Bienvenue
	ldi zh,high(BV2*2)	
	rcall Aff_Lg2

	ldi r16,0x30			; Pause de 3s environ pour bienvenue
b1:	ldi r17,0xFF
b2:	ldi r18,0xFF
b3:	dec r18
	brne b3
	dec r17
	brne b2
	dec r16
	brne b1

	ldi Zl,low(VIDE*2)		; Afficher la ligne 1 Ecran
	ldi Zh,high(VIDE*2)	
	rcall Aff_Lg1

	ldi Zl,low(VIDE*2)		; Afficher la ligne 2 Ecran
	ldi Zh,high(VIDE*2)	
	rcall Aff_Lg2

	;----------------- Autorisation des intrruptions ----------------------------------
	sei			; Autorisation des interruptions aprs message bienvenue
				; et mise en place ecran de travail

PRINC:	
	rjmp PRINC


;******************************************************************************************
;			ROUTINES D'INTERRUPTIONS
;******************************************************************************************
;______________________________________ TIMER1 COMPARAISON _____________________________
T1_COMP:				; Rgl sur 50ms
					; Gestion Heure et Clavier
	push temp		; Empiler temp
	in temp,SREG
	push temp		; Empiler SREG

	;sei			; Permet de traiter d'autres interruptions
				
	;----------------- Traitement du clavier ( ttes les 50ms) -------------------------
Clavier:
	rcall LitClavier	; Lecture du clavier (code Physique ou rel)
	cpi Touche,0x10
	breq Fin_Clavier
	cpi Touche,0x20
	breq Fin_Clavier
	sbrs DRAPEAUX,5		; test si OrdiON=1 saut si oui
	rjmp OrdiOff
att0:	in r16,USR
	andi r16,0x20		; Registre d'emission libre ?
	breq att0
	out UDR,Touche		; Envoi de l'octet haut du code
	rjmp Fin_Clavier
OrdiOff:
	ldi zl,low(ARRET1*2)	; Afficher la ligne 1 Bienvenue
	ldi zh,high(ARRET1*2)	
	rcall Aff_Lg1
	ldi zl,low(ARRET2*2)	; Afficher la ligne 2 Bienvenue
	ldi zh,high(ARRET2*2)	
	rcall Aff_Lg2
	ldi AfficheTps,100	; Afficher 5s
Fin_Clavier:

Test_Tps:
	tst AfficheTps		; AfficheTps=0 ?
	breq Fin_Test_Tps	; Si oui fin
	dec AfficheTps
	tst AfficheTps	
	brne Fin_Test_Tps	; Si pas 0 fin
	ldi zl,low(VIDE*2)	; Afficher la ligne 1 Vide
	ldi zh,high(VIDE*2)	
	rcall Aff_Lg1
	ldi zl,low(VIDE*2)	; Afficher la ligne 2 Vide
	ldi zh,high(VIDE*2)	
	rcall Aff_Lg2
Fin_Test_Tps:

Chien:
	tst T10		; Chien de garde abboie ?
	brne PasFini
	sbrs DRAPEAUX,5	; OrdiOn=1, si oui: saut
	rjmp Fin_Chien
	ldi zl,low(VIDE*2)	; Afficher la ligne 1 Vide
	ldi zh,high(VIDE*2)	
	rcall Aff_Lg1
	ldi zl,low(VIDE*2)	; Afficher la ligne 2 Vide
	ldi zh,high(VIDE*2)	
	rcall Aff_Lg2
	cbr DRAPEAUX,0x20	; OrdiOn=0
	rjmp Fin_Chien

PasFini:
	dec T10			; Dcrmenter chien

Fin_Chien:


Fin_Timer1:
	pop temp
	out SREG,temp		; Dpiler SREG
	pop temp		; Dpiler temp
	reti


;______________________________________ UART_RXC _________________________________________
UART_RXC:
	push temp		; Empiler temp
	in temp,SREG
	push temp		; Empiler SREG
	
	ldi T10,200		; Chien de garde 10s
	sbr DRAPEAUX,0x20	; OrdiOn=1

	in r16,UDR		; Lire donne

	tst r16			
	breq Fin_Rx		; Fin si Car OrdiOn (0x00)

	cpi r16,0x01		; Aller en ligne1 ?
	brne T2			; Non
	ldi r16,0x80
	rcall PROG_LCD
	rjmp Fin_Rx
T2:
	cpi r16,0x02		; Aller en ligne2 ?
	brne T3  		; Non
	ldi r16,0xC0
	rcall PROG_LCD
	rjmp Fin_Rx
T3:
	cpi r16,0x03		; Afficher curseur ?
	brne T4 		; Non
	rcall Curseur_On
	rjmp Fin_Rx
T4:
	cpi r16,0x04		; Eteindre Curseur ?
	brne Affiche		; Non
	rcall Curseur_Off
	rjmp Fin_Rx

Affiche:
	rcall DATA_LCD		; Affiche le caractre

Fin_Rx:	
	pop temp
	out SREG,temp		; Dpiler SREG
	pop temp		; Dpiler temp
	reti


;******************************************************************************************
;			SOUS PROGRAMMES
;******************************************************************************************
;_________________________ SOUS PROGRAMMES Gestion LCD _______________________________
.include	"LCDCLAV.sub"	; Routines de gestionnaire LCD + CLAVIER


;******************************************************************************************
;			TABLES DE CARACTERES, TABLE ROM.....
;******************************************************************************************
BV1:	.db "    Juke Box    ",0
BV2: 	.db "                ",0

ARRET1:	.db "   Ordinateur   ",0
ARRET2:	.db "     Eteint     ",0

VIDE:	.db "                ",0
